<?php

namespace App;

use DB;

//use Illuminate\Http\Request;
use Illuminate\Pagination\LengthAwarePaginator;
use Illuminate\Pagination\Paginator;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\DB as FacadesDB;
use Illuminate\Support\Str;
use Illuminate\Support\Facades\Request;


class Helpers 
{

    const STATUS_ENABLE = 1;
    const STATUS_DISABLE = 2;

    const SMS = 1;
    const EMAIL = 2;
    const PUSH = 3;

    const STATUSCODE_SUCCESS = 200;
    const STATUSCODE_FAILED = 400;
    const STATUSCODE_NOT_FOUND = 404;

    public static function get_slug_id($table, $slug)
    {
        $slug_details = FacadesDB::table($table)->where('slug', $slug)->select('id', 'program_name', 'status')->first();
        return $slug_details;
    }

    public static function lastinserted($table)
    {
        return FacadesDB::table($table)->orderBy('id', 'DESC')->first();
    }

    public static function statusNameFor($status)
    {
        switch($status) {
            case static::STATUS_ENABLE : return 'open';
            case static::STATUS_DISABLE : return 'disable';
        }
    }

    public static function medium($medium)
    {
        switch($medium) {
            case static::SMS : return 'sms';
            case static::EMAIL : return 'email';
            case static::PUSH : return 'push notification';
        }
    }

    public static function program($token)
    {
        $program = FacadesDB::table('programs')->where('token', $token)->select('program_name')->get();
        return $program;
    }

    

    public static function createSlug($title, $table)
    {
        //Normalize the title
        $slug = Str::slug($title);

        //Get any that could possibly be related
        //This cuts the queries down by doing it once

        $allSlugs = Static::getRelatedSlugs($slug, $table);

        //if we haven't used it before then we all good.
        if(! $allSlugs->contains('slug', $slug)){
            return $slug;
        }else{
            return false;
        }

      
    }

    protected static function getRelatedSlugs($slug, $table)
    {
        return FacadesDB::table($table)->where('slug', 'like', $slug. '%')
                        ->get();
    }

    public static function details($table, $id)
    {
        $details =  FacadesDB::table($table)->where('id', $id)->first();
        if(is_null($details)){
            return null;
        }else{
            return $details;
        }

    }

    public static function paginate($items, $perPage = 5, $page = null)
    {
        $page = $page ?: (Paginator::resolveCurrentPage() ?: 1);
        $items = $items instanceof Collection ? $items : Collection::make($items);
        return new LengthAwarePaginator($items->forPage($page, $perPage), $items->count(),$perPage, $page, ['path'=> Paginator::resolveCurrentPath(), 'pageName'=>'page']);
    }

    public static function addToLog($subject, $slug = null, $new_data = null, $old_data = null)
    {
        $log = [];
        if(is_null($old_data)){
            $olddata = "";
        }else{
            $olddata = serialize($old_data);
        }
        
        $log['subject'] = $subject;
        $log['url'] = Request::fullUrl();
        $log['method'] = Request::method();
        $log['ip'] = Request::ip();
        $log['agent'] = Request::header('user-agent');
        $log['slug'] = $slug;
        $log['old_data'] = $olddata;
        $log['new_data'] = serialize($new_data);
        // $log['old_data'] = serialize(collect($old_data));
        // $log['new_data'] = serialize(collect($new_data));

        FacadesDB::table('log_activity')->insert($log);
    }

    public static function Auth()
    {
        $token = Request::header('token');
        return FacadesDB::table('programs')->where('token', $token)->first();
    }

   

   
    

}