<?php

namespace App\Http\Controllers;

use App\Helpers;
use App\Models\Location;
use DB;
use Illuminate\Http\Request;


class LocationController extends Controller 
{
    protected $perPage = 20;

    public function all(Request $request, $status = 1)
    {
        if(isset($request['limit'])){
            $limit = $request['limit'];
        }else{
            $limit = $this->perPage;
        }
        $program = DB::table('programs')->where('token', $request->header('token'))->first();
        $locations = Location::all($status, $limit, $program->id);
        return response()->json($locations);
    }

    public function store(Request $request)
    {
        $this->validate($request, [
            'loyalty_program_id'=> 'required',
            'location_name'=> 'required|string|max:255',
            'location_slug' => 'required',
            'longitude'=> 'required|numeric|between:-180,180',
            'latitude'=> 'required|numeric|between:-180,180',
            'radius'=> 'required',
        ]);

        $create_slug = Helpers::createSlug($request['location_slug'], 'locations');
        $check_program = DB::table('programs')->where('id', $request['loyalty_program_id'])->exists();
        if($create_slug == false){
            return response()->json(['message'=> 'location identifier already exists', 'status'=> 0, 'status_code'=> 400], 400);
         }elseif(!$check_program){
            return response()->json(['message'=> 'Program not found', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_NOT_FOUND], Helpers::STATUSCODE_NOT_FOUND);
         }else{
             $data = [
                 'program_id'=> $request['loyalty_program_id'],
                'location_name'=> $request['location_name'],
                'slug' => $create_slug,
                'longitude' => $request['longitude'],
                'latitude'=> $request['latitude'],
                'radius'=> $request['radius']
             ];

             $store = Location::create_location($data);
             if($store){
                 $location = Helpers::lastinserted('locations');
                 $data = [
                     'loyalty_program_id'=> $location->program_id,
                    'location_name'=> $location->location_name,
                    'location_slug'=> $location->slug,
                    'longitude'=> $location->longitude,
                    'latitude'=> $location->latitude,
                    'status'=> $location->status,
                    'created_at'=> $location->created_at,
                    'updated_at'=> $location->updated_at,
                 ];
                 //Log activity
                 Helpers::addToLog('Location Created', $create_slug, $data);

                 return response()->json(['message'=> 'Location created successfully', 'status'=>1, 'status_code'=> Helpers::STATUSCODE_SUCCESS, 'data'=> $data]);
             }else{
                 return response()->json(['message'=> 'Invalid request', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_FAILED]);
             }
         }

    }

    public function update(Request $request, $location_slug)
    {
        $check_slug = Location::location_details($location_slug);
        if(is_null($check_slug)){
            return response()->json(['message'=> 'location not found', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_NOT_FOUND]);
        }elseif($check_slug->status == 2){
            return response()->json(['message'=> 'location currently not active', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_FAILED]);
        }else{
           $update = Location::update_location($check_slug->id, $request->only(['location_name', 'longitude', 'latitude', 'status', 'radius']));
           if($update){
            $location = Location::find_location($check_slug->id);
            $data = [
                'location_name'=> $location->location_name,
                'location_slug' => $location->slug,
                'longitude'=> $location->longitude,
                'latitude'=> $location->latitude,
                'status'=> $location->status,
                'created_at'=> $location->created_at,
                'updated_at' => $location->updated_at
            ];
             //Log activity
             Helpers::addToLog('Updated Location', $location_slug, $request->only(['location_name', 'longitude', 'latitude', 'status']), $check_slug);
            return response()->json(['message'=> 'Location Updated Successfully', 'status'=> 1, 'status_code'=> Helpers::STATUSCODE_SUCCESS, 'data'=> $data]);
           }else{
            return response()->json(['message'=> 'Invalid request', 'status'=>0, 'status_code'=> Helpers::STATUSCODE_FAILED], Helpers::STATUSCODE_FAILED);
           }
        }
    }

    public function show($location_slug)
    {
        $check_slug = Location::location_details($location_slug);
        if(is_null($check_slug)){
            return response()->json(['message'=> 'location not found', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_NOT_FOUND],  Helpers::STATUSCODE_NOT_FOUND); 
        }else{
            $location = Location::find_location($check_slug->id);
            $data = [
                'location_name'=> $location->location_name,
                'location_slug' => $location->slug,
                'longitude'=> $location->longitude,
                'latitude'=> $location->latitude,
                'status'=> $location->status,
                'created_at'=> $location->created_at,
                'updated_at' => $location->updated_at
            ];
            return response()->json(['status'=> 1, 'status_code'=>Helpers::STATUSCODE_SUCCESS, 'data'=> $data], Helpers::STATUSCODE_SUCCESS);
        }
    }

    public function disable($location_slug)
    {
        $check_slug = Location::location_details($location_slug);
        if(is_null($check_slug)){
            return response()->json(['message'=> 'location not found', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_NOT_FOUND],  Helpers::STATUSCODE_NOT_FOUND); 
        }elseif($check_slug->status == 2){
            return response()->json(['message'=> 'location currently disabled', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_FAILED], Helpers::STATUSCODE_FAILED);
        }else{
            $data['status'] = Helpers::STATUS_DISABLE;
            $update = Location::update_location($check_slug->id, $data);
            if($update){
                Helpers::addToLog('Disable location', $location_slug, $data, $check_slug);
                return response()->json(['message'=> 'Location disabled successfully', 'status'=> 1, 'status_code'=> Helpers::STATUSCODE_SUCCESS], Helpers::STATUSCODE_SUCCESS);
            }else{
                return response()->json((['message'=> 'Error', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_FAILED]), Helpers::STATUSCODE_FAILED);
            }
        }
    }

    public function enable($location_slug)
    {
        $check_slug = Location::location_details($location_slug);
        if(is_null($check_slug)){
            return response()->json(['message'=> 'location not found', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_NOT_FOUND],  Helpers::STATUSCODE_NOT_FOUND); 
        }elseif($check_slug->status == 1){
            return response()->json(['message'=> 'location currently active', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_FAILED], Helpers::STATUSCODE_FAILED);
        }else{
            $data['status'] = Helpers::STATUS_ENABLE;
            $update = Location::update_location($check_slug->id, $data);
            if($update){
                Helpers::addToLog('Enable location', $location_slug, $data, $check_slug);
                return response()->json(['message'=> 'Location Enabled successfully', 'status'=> 1, 'status_code'=> Helpers::STATUSCODE_SUCCESS], Helpers::STATUSCODE_SUCCESS);
            }else{
                return response()->json((['message'=> 'Error', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_FAILED]), Helpers::STATUSCODE_FAILED);
            }
        }
    }

    //Assign location to program
    public function create_branch(Request $request)
    {
        $this->validate($request, [
            'loyalty_program_id'=> 'required|integer',
            'location_id'=> 'required|integer',
        ]);

        //check if branch exists
        $check_location = Location:: programlocation_exists($request['loyalty_program_id'],$request['location_id']);
        if($check_location){
            return response()->json(['message'=> 'Location Already Been Assigned to this program', 'status'=>0, 'status_code'=> Helpers::STATUSCODE_FAILED], Helpers::STATUSCODE_FAILED);
        }else{
            $data = [
                'loyalty_program_id'=> $request['loyalty_program_id'],
                'location_id'=> $request['location_id']
            ];
            
            $assign = Location::location_to_program($data);
            if($assign){
                $lastinserted = Helpers::lastinserted('program_locations');
                $program = Helpers::details('programs', $request['loyalty_program_id']);
                $location = Helpers::details('locations', $request['location_id']);
                $res = [
                    'program'=> $program->program_name,
                    'location'=> $location->location_name,
                    'status'=> $lastinserted->status
                ];
                Helpers::addToLog('Program Location Created', $program->slug, $data);
                return response()->json(['message'=> 'Location Assigned Successfully', 'status'=> 1, 'status_code'=> Helpers::STATUSCODE_SUCCESS, 'data'=> $res], Helpers::STATUSCODE_SUCCESS); 
            }else{
                return response()->json(['message'=> 'Invalid request', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_FAILED],Helpers::STATUSCODE_FAILED);
            }
        }
        
       
    }

    public function update_branch(Request $request, $location_slug)
    {
        $check_slug = Location::location_details($location_slug);
        if(is_null($check_slug)){
            return response()->json(['message'=> 'location not found', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_NOT_FOUND],  Helpers::STATUSCODE_NOT_FOUND); 
        }else{
            $program_location = Location::find_branch($check_slug->id);
            if($program_location){
                $update = Location::update_branch($program_location->id, $request->only(['loyalty_program_id', 'status']));
                if($update){
                    $res = Location::find_branch($program_location->id);
                    $program = Helpers::details('programs', $res->loyalty_program_id);
                    $location = Helpers::details('locations', $res->location_id);
                    $data = [
                        'program'=> $program->program_name,
                        'location'=> $location->location_name,
                    ];
                    Helpers::addToLog('Program Location Updated', $location_slug, $request->only(['loyalty_program_id', 'status']), $check_slug);
                    return response()->json(['message'=> 'updated successfully', 'status'=> 1, 'status_code'=>Helpers::STATUSCODE_SUCCESS, 'data'=> $data], Helpers::STATUSCODE_SUCCESS);
                }else{
                    return response()->json(['message'=> 'Invalid request', 'status'=>0, 'status_code'=> Helpers::STATUSCODE_FAILED], Helpers::STATUSCODE_FAILED);
                }
            }else{
                return response()->json(['message'=> 'location not assign to any progam', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_NOT_FOUND],  Helpers::STATUSCODE_NOT_FOUND); 
            }
        }
    }

    public function find_branch($location_slug)
    {
        $check_slug = Location::location_details($location_slug);
        if(is_null($check_slug)){
            return response()->json(['message'=> 'location not found', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_NOT_FOUND],  Helpers::STATUSCODE_NOT_FOUND);
        }else{
            $branches = Location::find_branch($check_slug->id);
            if($branches){
                $program = Helpers::details('programs', $branches->loyalty_program_id);
                $location = Helpers::details('locations', $branches->location_id);
                $data = [
                    'program'=> $program->program_name,
                    'location'=> $location->location_name,
                    'status'=> $branches->status,
                    'created_at'=> $branches->created_at,
                    'updated_at'=> $branches->updated_at,
                ];
                
                return response()->json($data);
            }else{
                return response()->json(['message'=> 'Program Location not found', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_NOT_FOUND],  Helpers::STATUSCODE_NOT_FOUND); 
            }
        }
    }

    public function branches($status = null)
    {
       $program = [];
       $location = [];
       $programlocation = [];
        $branches = Location::all_branches($status);
        for ($i=0; $i < count($branches); $i++) { 
            
            array_push($program, Helpers::details('programs',$branches[$i]->loyalty_program_id));
            array_push($location,  Helpers::details('locations', $branches[$i]->location_id));  
            $data = [
                'id'=> $branches[$i]->id,
                'program'=>$program[$i]->program_name,
                'location'=> $location[$i]->location_name,
                'status'=> $branches[$i]->status,
                'created_at'=> $branches[$i]->created_at,
                'updated_at'=> $branches[$i]->updated_at,
            ];
            array_push($programlocation, $data);
            $fontdata = Helpers::paginate($programlocation, $this->perPage); 
        }
        return response()->json($fontdata);
    }

    public function delete_branch($location_slug)
    {
        $check_slug = Location::location_details($location_slug);
        if(is_null($check_slug)){
            return response()->json(['message'=> 'location not found', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_NOT_FOUND],  Helpers::STATUSCODE_NOT_FOUND);
        }else{
            $delete = Location::delete_branch($check_slug->id);
            if($delete){
                Helpers::addToLog('Program Location Deleted', $location_slug);
                return response()->json(['message'=> 'Deleted successfully', 'status'=> 1, 'status_code'=> Helpers::STATUSCODE_SUCCESS], Helpers::STATUSCODE_SUCCESS);
            }else{
                return response()->json(['message'=> 'Error', 'status'=> 1, 'status_code'=> Helpers::STATUSCODE_FAILED], Helpers::STATUSCODE_FAILED);
            }
        }
    }
}