<?php

namespace App\Http\Controllers;

use App\Helpers;
use App\Models\Location;
use App\Models\Message;
use App\Models\Program;
use DB;
use Illuminate\Http\Request;

class MessageController extends Controller 
{

    protected $perPage = 20;

    public function all(Request $request, $status = null)
    {
        $messagedata = [];
        $program = [];
        if(isset($request['limit'])){
            $limit = $request['limit'];
        }else{
            $limit = $this->perPage;
        }
        $program_id = DB::table('programs')->where('token', $request->header('token'))->select('id')->first();
        $res = Message::all_messages($status, $program_id->id);
        
        for ($i=0; $i < count($res) ; $i++) { 
            array_push($program, Program::find($res[$i]->loyalty_program_id));
            //$location = Helpers::details('location_message',$res[$i]->location_id);
            $data = [
                'id'=> $res[$i]->id,
                'program'=> $program[$i]->program_name,
                'message_name'=> $res[$i]->message_name,
                'message_body'=> $res[$i]->message_body,
                'priority'=> $res[$i]->priority,
                'status'=> $res[$i]->status,
                'created_at'=> $res[$i]->created_at,
                'updated_at'=> $res[$i]->updated_at,
             ];
            array_push($messagedata, $data);
            $message = Helpers::paginate($messagedata, $limit); 
        }
        return response()->json($message);
    }

    public function store(Request $request)
    {
        $this->validate($request, [
            'loyalty_program_id'=> 'required|integer',
            'message_name'=> 'required|string',
            'message_body'=> 'required|string',
            'priority' => 'required|integer',
        ]);
        //$program_id = DB::table('programs')->where('token', $request->header('token'))->select('id')->first();
        $check_message = DB::table('messages')->where('message_name', $request['message_name'])->exists();
        $check_program = DB::table('programs')->where('id', $request['loyalty_program_id'])->exists();
        $priority_check = DB::table('messages')->where('loyalty_program_id', $request['loyalty_program_id'])->where('priority', $request['priority'])->exists();
        if($check_message){
            return response()->json(['message'=> 'Message Already Exists', 'status'=> 0, 'status_code'=> 400], 400);
        }elseif(!$check_program){
            return response()->json(['message'=> 'Loyalty Program not found', 'status'=> 0, 'status_code'=> 404], 404);
        }elseif($priority_check){
            return response()->json(['message'=> 'Message with priority already exists', 'status'=> 0, 'status_code'=> 400], 400);
        }else{
            $data = [
                'loyalty_program_id'=> $request['loyalty_program_id'],
                'message_name'=> $request['message_name'],
                'message_body'=> $request['message_body'],
                'priority'=> $request['priority']
            ];
            $store = Message::create_message($data);
            if($store){
                $message = Helpers::lastinserted('messages');
                $program = Program::find($message->loyalty_program_id);
                //$location = Helpers::details('locations',$message->location_id);
                //Log recent activity
                Helpers::addToLog('New Message Created', $program->slug, $data);

                $messagedata = [
                   'program'=> $program->program_name,
                   //'location'=> $location,
                   'message_name'=> $message->message_name,
                   'message_body'=> $message->message_body,
                   'priority'=> $message->priority,
                   'status'=> $message->status,
                   'created_at'=> $message->created_at,
                   'updated_at'=> $message->updated_at,
                ];

                return response()->json(['message'=> 'Message created successfully', 'status'=>1, 'status_code'=> Helpers::STATUSCODE_SUCCESS, 'data'=> $messagedata]);
            }else{
                return response()->json(['message'=> 'Invalid request', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_FAILED]);
            }
        }
       


    }

    public function show($message_id)
    {
        $check_id = Message::show_message($message_id);
        if(is_null($check_id)){
            return response()->json(['message'=> 'Message not found', 'status'=> 0, 'status_code'=>Helpers::STATUSCODE_NOT_FOUND],Helpers::STATUSCODE_NOT_FOUND );
        }else{
            $message = Message::show_message($check_id->id);
            $program = Program::find($message->loyalty_program_id);
            $location = Helpers::details('locations',$message->location_id);
            $data = [
                'program'=> $program->program_name,
                'location'=> $location->location_name,
                'message_name'=> $message->message_name,
                'message_body'=> $message->message_body,
                'priority'=> $message->priority,
                'status'=> $message->status,
                'created_at'=> $message->created_at,
                'updated_at'=> $message->updated_at,
             ];

             return response()->json(['status'=> 1, 'status_code'=>Helpers::STATUSCODE_SUCCESS, 'data'=> $data], Helpers::STATUSCODE_SUCCESS);

        }

    }

    public function update(Request $request, $message_id)
    {
        $check_id = Message::show_message($message_id);
        if(is_null($check_id)){
            return response()->json(['message'=> 'Message not found', 'status'=> 0, 'status_code'=>Helpers::STATUSCODE_NOT_FOUND],Helpers::STATUSCODE_NOT_FOUND );
        }elseif($check_id->status == 2){
            return response()->json(['message'=> 'Message is down', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_FAILED]);
        }else{
            $update = Message::update_message($check_id->id, $request->only(['loyalty_program_id','location_id', 'message_name', 'message_body', 'priority', 'status' ]));
            if($update){
                $message = Message::show_message($check_id->id);
                $program = Program::find($message->loyalty_program_id);
                $location = Helpers::details('locations',$message->location_id);
                //Log recent activity
                Helpers::addToLog('Message Updated', $message_id, $request->only(['loyalty_program_id', 'message_type', 'message_name', 'message_body', 'priority', 'status' ]), $check_id);
                $data = [
                    'program'=> $program->program_name,
                    'message_name'=> $message->message_name,
                    'location'=> $location->location_name,
                    'message_body'=> $message->message_body,
                    'priority'=> $message->priority,
                    'status'=> $message->status,
                    'created_at'=> $message->created_at,
                    'updated_at'=> $message->updated_at,
                ];
                return response()->json(['message'=> 'Message updated successfully', 'status'=> 1, 'status_code'=> Helpers::STATUSCODE_SUCCESS, 'data'=> $data],Helpers::STATUSCODE_SUCCESS);
            }else{
                return response()->json(['message'=> 'Invalid request', 'status'=>0, 'status_code'=> Helpers::STATUSCODE_FAILED], Helpers::STATUSCODE_FAILED);
            }
        }
    }

    public function disable($message_id)
    {
        $check_id = Message::show_message($message_id);
        if(is_null($check_id)){
            return response()->json(['message'=> 'Message not found', 'status'=> 0, 'status_code'=>Helpers::STATUSCODE_NOT_FOUND],Helpers::STATUSCODE_NOT_FOUND );
        }elseif($check_id->status == 2){
            return response()->json(['message'=> 'Message currently disabled', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_FAILED], Helpers::STATUSCODE_FAILED);
        }else{
            $data['status'] = Helpers::STATUS_DISABLE;
            $update = Message::update_message($check_id->id, $data);
            if($update){
                //Log recent activity
                Helpers::addToLog('Message Disabled', $message_id, $data, $check_id);
                return response()->json(['message'=> 'Message disabled successfully', 'status'=> 1, 'status_code'=> Helpers::STATUSCODE_SUCCESS], Helpers::STATUSCODE_SUCCESS);
            }else{
                return response()->json((['message'=> 'Error', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_FAILED]), Helpers::STATUSCODE_FAILED);
            }
        }
    }

    public function enable($message_id)
    {
        $check_id = Message::show_message($message_id);
        if(is_null($check_id)){
            return response()->json(['message'=> 'Message not found', 'status'=> 0, 'status_code'=>Helpers::STATUSCODE_NOT_FOUND],Helpers::STATUSCODE_NOT_FOUND );
        }elseif($check_id->status == 1){
            return response()->json(['message'=> 'Message currently active', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_FAILED], Helpers::STATUSCODE_FAILED);
        }else{
            $data['status'] = Helpers::STATUS_ENABLE;
            $update = Message::update_message($check_id->id, $data);
            if($update){
                 //Log recent activity
                 Helpers::addToLog('Message Enabled', $message_id, $data, $check_id);
                return response()->json(['message'=> 'Message Enabled successfully', 'status'=> 1, 'status_code'=> Helpers::STATUSCODE_SUCCESS], Helpers::STATUSCODE_SUCCESS);
            }else{
                return response()->json((['message'=> 'Error', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_FAILED]), Helpers::STATUSCODE_FAILED);
            }
        }
    }

    public function assign_to_location(Request $request)
    {
        $this->validate($request, [
            'location_id'=> 'required|integer',
            'message_id'=> 'required|integer',
        ]);
        
        $check_location = Message::LocationMessage_Exist($request['location_id'], $request['message_id']);
        $check_location_status = DB::table('locations')->where('id', $request['location_id'])->select('status')->first();
        $program_id = DB::table('programs')->where('token', $request->header('token'))->select('id')->first();
        $program_location_check = DB::table('locations')->where('program_id', $program_id->id)->where('id', $request['location_id'])->exists();
        $program_message_check = DB::table('messages')->where('loyalty_program_id', $program_id->id)->where('id', $request['message_id'])->exists();
        if($check_location){
            return response()->json(['message'=> 'Message Already Assigned to Location', 'status'=>0, 'status_code'=> Helpers::STATUSCODE_FAILED], Helpers::STATUSCODE_FAILED);
        }elseif($check_location_status->status != 1){
            return response()->json(['message'=> 'Message cannot be assigned to this location', 'status'=> 0, 'status_code'=>Helpers::STATUSCODE_FAILED], Helpers::STATUSCODE_FAILED);
        }elseif(!$program_location_check){
            return response()->json(['message'=> 'location does not belong to program', 'status'=> 0, 'status_code'=>Helpers::STATUSCODE_FAILED], Helpers::STATUSCODE_FAILED);
        }elseif(!$program_message_check){
            return response()->json(['message'=> 'message does not belong to program', 'status'=> 0, 'status_code'=>Helpers::STATUSCODE_FAILED], Helpers::STATUSCODE_FAILED);
        }else{
            $location = Helpers::details('locations', $request['location_id']);
            $message = Helpers::details('messages', $request['message_id']);
            $data = [
                'location_id'=>$request['location_id'],
                'message_id'=> $request['message_id'],
                'priority'=> $message->priority
            ];
            $store = Message::Message_to_Location($data);
            //$data['location_id'] = $request['location_id'];
            //$store = Message::update_message($request['message_id'], $data);
            if($store){
                //$lastinserted = Helpers::lastinserted('location_message');
                
                $res = [
                    'location'=> [
                        'location_name'=>$location->location_name,
                    ],
                    'message'=> [
                       'message_name'=> $message->message_name,
                       'message_body'=> $message->message_body,
                    ],
                    // 'status'=> $lastinserted->status
                ];
                Helpers::addToLog('Location message created','',$data);
                return response()->json(['message'=> 'Message Assigned Successfully', 'status'=>1, 'status_code'=> Helpers::STATUSCODE_SUCCESS, 'data'=>$res], Helpers::STATUSCODE_SUCCESS);
    
            }else{
                return response()->json(['message'=> 'Invalid request', 'status'=>0, 'status_code'=> Helpers::STATUSCODE_FAILED], Helpers::STATUSCODE_FAILED);
            }
        }
       
    }

    public function update_location_message(Request $request, $location_slug, $message_id)
    {
        $check_slug = Location::location_details($location_slug);
        //$check_message_id = DB::table('location_message')->where('location_id', $check_slug->id)->where('message_id', $message_id)->exists();
        if(is_null($check_slug)){
            return response()->json(['message'=> 'location not found', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_NOT_FOUND],  Helpers::STATUSCODE_NOT_FOUND); 
        }else{
            $location_message = Message::show_location_message($check_slug->id);
            if($location_message){
                $update = Message::update_location_message($location_message->id, $request->only(['message_id', 'status']));
                if($update){
                    $updated_location_message = Message::show_location_message($check_slug->id);
                    $location = Location::find_location($updated_location_message->location_id);
                    $message = Message::show_message($updated_location_message->message_id);
                    
                    $res = [
                        'location'=> [
                            'location_name'=>$location->location_name,
                            'location_slug'=> $location->slug,
                        ],
                        'message'=> [
                           'message_name'=> $message->message_name,
                           'message_slug'=> $message->slug,
                           'message_body'=> $message->message_body
                        ],
                        'status'=> $updated_location_message->status
                    ];
                    //Log activity
                    Helpers::addToLog('Updated a location message',$location_slug,$request->only(['message_id', 'status']), $location_message);
                    return response()->json(['message'=> 'updated successfully', 'status'=> 1, 'status_code'=>Helpers::STATUSCODE_SUCCESS, 'data'=> $res], Helpers::STATUSCODE_SUCCESS);
                }else{
                    return response()->json(['message'=> 'Invalid request', 'status'=>0, 'status_code'=> Helpers::STATUSCODE_FAILED], Helpers::STATUSCODE_FAILED);
                }
            }else{
                return response()->json(['message'=> 'location not assign to any progam', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_NOT_FOUND],  Helpers::STATUSCODE_NOT_FOUND); 
            }

        }
    }

    public function find_location_message($location_slug, $message_id)
    {
        $check_slug = Location::location_details($location_slug);
        if(is_null($check_slug)){
            return response()->json(['message'=> 'location not found', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_NOT_FOUND],  Helpers::STATUSCODE_NOT_FOUND); 
        }else{
            $check_message = DB::table('location_message')->where('location_id', $check_slug->id)->where('message_id', $message_id)->exists();
            if($check_message){
                $message = Message::show_message($message_id);
                $res = [
                        'messaga_id'=> $message->id,
                        'message_name'=> $message->message_name,
                        'message_body'=> $message->message_body,
                        'message_priority'=> $message->priority
                ];
                return response()->json(['data'=> $res, 'status'=>1, 'status_code'=> Helpers::STATUSCODE_SUCCESS], Helpers::STATUSCODE_SUCCESS);
            }else{
                return response()->json(['message'=> 'message not assign to the requested location', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_FAILED],  Helpers::STATUSCODE_FAILED); 
            }
        }   
       
    }

    public function location_message($location_slug)
    {
        $check_slug = Location::location_details($location_slug);
         if(is_null($check_slug)){
            return response()->json(['message'=> 'location not found', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_NOT_FOUND],  Helpers::STATUSCODE_NOT_FOUND); 
        }else{
            $results = [];
            $location_message = Message::show_location_message($check_slug->id);
            if($location_message){

                foreach ($location_message as $message) {
                   // $location = Location::find_location($message->location_id);
                    $message = Message::show_message($message->message_id);
                    
                    $res = [
                        'message_id'=> $message->id,
                        'message_name'=> $message->message_name,
                        'message_body'=> $message->message_body,
                        'message_priority'=> $message->priority
                        
                       
                    ];

                    array_push($results, $res);
                }
               
                return response()->json($results);
            }else{
                return response()->json(['message'=> 'No Messages found for this location', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_NOT_FOUND],  Helpers::STATUSCODE_NOT_FOUND); 
            }
        }
    }

    public function all_location_message(Request $request, $status = null)
    {
        $locations = [];
        $messages = [];
        $locationmessages =  [];
        if(isset($request['limit'])){
            $limit = $request['limit'];
        }else{
            $limit = $this->perPage;
        }
        $location_message = Message::all_location_messages($status);
        for ($i=0; $i < count($location_message); $i++) { 
            
            array_push($locations, Helpers::details('locations',$location_message[$i]->location_id));
            array_push($messages,  Helpers::details('messages', $location_message[$i]->message_id));  
            $res = [
                'location'=> [
                    'location_id'=> $locations[$i]->id,
                    'location_name'=>$locations[$i]->location_name,
                ],
                'message'=> [
                    'message_id'=> $messages[$i]->id,
                   'message_name'=> $messages[$i]->message_name,
                   'message_body'=> $messages[$i]->message_body
                ],
                'status'=> $location_message[$i]->status
            ];
            array_push($locationmessages, $res);
            $data = Helpers::paginate($locationmessages, $limit); 
        }
        return response()->json($data);

    }

    public function disable_location_message($location_slug)
    {
        $check_slug = Location::location_details($location_slug);
        if(is_null($check_slug)){
            return response()->json(['message'=> 'location not found', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_NOT_FOUND],  Helpers::STATUSCODE_NOT_FOUND);
        }else{
            $location_message = Message::show_location_message($check_slug->id);
            if($location_message->status == 2){
                return response()->json(['message'=> 'Message currently disabled', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_FAILED], Helpers::STATUSCODE_FAILED);
            }else{
                $data['status'] = Helpers::STATUS_DISABLE;
                $update = Message::update_location_message($location_message->id, $data);
                if($update){
                    //Log activity
                    Helpers::addToLog('Disable location message', $location_slug, $data, $location_message);
                    return response()->json(['message'=> 'Location Message Disabled successfully', 'status'=> 1, 'status_code'=> Helpers::STATUSCODE_SUCCESS], Helpers::STATUSCODE_SUCCESS);
                }else{
                    return response()->json(['message'=> 'Error', 'status'=>0, 'status_code'=> Helpers::STATUSCODE_FAILED],Helpers::STATUSCODE_FAILED);
                }
            }
        }

    }

    public function enable_location_message($location_slug)
    {
        $check_slug = Location::location_details($location_slug);
        if(is_null($check_slug)){
            return response()->json(['message'=> 'location not found', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_NOT_FOUND],  Helpers::STATUSCODE_NOT_FOUND);
        }else{
            $location_message = Message::show_location_message($check_slug->id);
            if($location_message->status == 1){
                return response()->json(['message'=> 'Message currently active', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_FAILED], Helpers::STATUSCODE_FAILED);
            }else{
                $data['status'] = Helpers::STATUS_ENABLE;
                $update = Message::update_location_message($location_message->id, $data);
                if($update){
                     //Log activity
                     Helpers::addToLog('Enable location message', $location_slug, $data, $location_message);
                    return response()->json(['message'=> 'Location Message Enabled successfully', 'status'=> 1, 'status_code'=> Helpers::STATUSCODE_SUCCESS], Helpers::STATUSCODE_SUCCESS);
                }else{
                    return response()->json(['message'=> 'Error', 'status'=>0, 'status_code'=> Helpers::STATUSCODE_FAILED],Helpers::STATUSCODE_FAILED);
                }
            }
        }

    }
}