<?php

namespace App\Http\Controllers;

use App\Helpers;
use App\Models\Program;
use App\Models\Program_Setting;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class ProSettingsController extends Controller 
{

    protected $perPage = 20;

    public function settings(Request $request, $status = null)
    {
        
        if(isset($request['limit'])){
            $limit = $request['limit'];
        }else{
            $limit = $this->perPage;
        }

        $res = Program_Setting::all($status, $limit);  
    
        return response()->json($res);
    }

    public function program_setting(Request $request)
    {
        $this->validate($request, [
            'loyalty_program'=> 'required',
            'maximum_distance' => 'required|numeric',
            'message_interval' => 'required|numeric',
            'message_medium' => 'required',
        ]);

        $check_program_slug = Program::find($request['loyalty_program']);
        if(is_null($check_program_slug)){
            return response()->json(['message'=> 'Program not found', 'status'=> 0, 'status_code'=>Helpers::STATUSCODE_NOT_FOUND]);
        }else{
            $setting_exists = DB::table('program_settings_master')->where('loyalty_program_id', $request['loyalty_program'])->exists();
            if($setting_exists){
                return response()->json(['message'=> 'Program setting already exists', 'status'=> 0, 'status_code'=> 419]);
            }else{
                if(is_array($request['message_medium'])){
                    $msg_medium = implode(',', $request['message_medium']);
                }else{
                    $msg_medium = $request['message_medium'];
                }
                $data = [
                    'loyalty_program_id'=> $request['loyalty_program'],
                    'maximum_distance'=> $request['maximum_distance'],
                    'message_interval'=> $request['message_interval'],
                    'message_medium' => $msg_medium,
                    'program_name' => $check_program_slug->program_name,
                ];
                
                $store = Program_Setting::create($data);
                if($store){
                    //Log the recent activity
                    Helpers::addToLog('created new program settings',$check_program_slug->slug, $data);

                    $program = Helpers::lastinserted('program_settings_master');
                    $programdata = [
                        'id'=> $program->id,
                        'program'=> $program->program_name,
                        'maximum_distance'=> $program->maximum_distance,
                        'message_interval' => $program->message_interval,
                        'message_medium'=> $program->message_medium,
                        'status'=> $program->status,
                        'created_at'=> $program->created_at,
                    ];
                    return response()->json(['message'=> 'Program setting saved successfully', 'status'=> 1, 'status_code'=> Helpers::STATUSCODE_SUCCESS, 'data'=> $programdata]);
                }else{
                    return response()->json(['message'=> 'Invaild request', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_FAILED]);
                }
            }
            
        }
        
    }


    public function update_program_setting(Request $request, $program_slug)
    {
        $check_program_slug = Helpers::get_slug_id('programs', $program_slug);
        if(is_null($check_program_slug)){
            return response()->json(['message'=> 'Program not found', 'status'=> 0, 'status_code'=>Helpers::STATUSCODE_NOT_FOUND]);
        }else{
            $settingdata = Program_Setting::find($check_program_slug->id);
            if($settingdata->status == 2){
                return response()->json(['message' => 'Program setting currently disabled', 'status'=>0, 'status_code'=> Helpers::STATUSCODE_FAILED ]);
            }else{
                $update = Program_Setting::update($settingdata->id, $request->only(['maximum_distance', 'message_interval','message_medium']));
                if($update){
                    //Log the recent activity
                    Helpers::addToLog('updated program setting', $program_slug, $request->only(['maximum_distance', 'message_interval','message_medium']), $settingdata);

                    $program = Program_Setting::find($check_program_slug->id);
                    $programdata = [
                        'program'=> $program->program_name,
                        'maximum_distance'=> $program->maximum_distance,
                        'message_interval' => $program->message_interval,
                        'message_medium'=> $program->message_medium,
                        'status'=> $program->status,
                        'created_at'=> $program->created_at,
                        'updated_at'=> $program->updated_at,
                    ];
                    return response()->json(['message'=>'Program setting updated successfully', 'status'=> 1, 'status_code'=> Helpers::STATUSCODE_SUCCESS, 'data'=>$programdata]);
                }else{
                    return response()->json(['message'=> 'Invaild request', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_FAILED]);
                }
            }
            
        }
    }

    public function find_program_setting($program_slug)
    {
        $check_program_slug = Helpers::get_slug_id('programs', $program_slug);
        if(is_null($check_program_slug)){
            return response()->json(['message'=> 'Program not found', 'status'=> 0, 'status_code'=>Helpers::STATUSCODE_NOT_FOUND]);
        }else{
            $program = Program_Setting::find($check_program_slug->id);
            if($program){
                $programdata = [
                    'program'=> $program->program_name,
                    // 'maximum_distance' => Helpers::details('max_distance', $program->max_distance),
                    // 'message_interval'=> Helpers::details('msg_interval', $program->msg_interval),
                    'maximum_distance'=> $program->maximum_distance,
                    'message_interval' => $program->message_interval,
                    'message_medium'=> $program->message_medium,
                    'status'=> $program->status,
                    'created_at'=> $program->created_at,
                ];
                return response()->json($programdata);
            }else{
                return response()->json(['message'=> 'Program setting not found', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_NOT_FOUND]);
            }
           
        }
    }

    public function disable_program_setting($program_slug)
    {
        $check_program_slug = Helpers::get_slug_id('programs', $program_slug);
        if(is_null($check_program_slug)){
            return response()->json(['message'=> 'Program not found', 'status'=> 0, 'status_code'=>Helpers::STATUSCODE_NOT_FOUND]);
        }else{
            $settingdata = Program_Setting::find($check_program_slug->id);
            if($settingdata){
                $data['status'] = Helpers::STATUS_DISABLE;
                $update = Program_Setting::update($settingdata->id, $data);
                if($update){
                    //Log recent activity
                    Helpers::addToLog('Disable program setting', $program_slug, $data, $settingdata);
                    return response()->json(['message'=>'Program Disabled Successfully', 'status'=> 1, 'status_code'=> Helpers::STATUSCODE_SUCCESS]);
                }else{
                    return response()->json(['message'=> 'Error', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_FAILED]);
                }
            }elseif($settingdata->status == 2){
                return response()->json(['message'=> 'Program is currently disabled', 'status'=> 0, 'status_code'=>Helpers::STATUSCODE_FAILED]);
            }else{
                return response()->json(['message'=> 'Program setting not found', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_NOT_FOUND]);
            }
           
        }
    }

    public function enable_program_setting($program_slug)
    {
        $check_program_slug = Helpers::get_slug_id('programs', $program_slug);
        if(is_null($check_program_slug)){
            return response()->json(['message'=> 'Program not found', 'status'=> 0, 'status_code'=>Helpers::STATUSCODE_NOT_FOUND]);
        }else{
            $settingdata = Program_Setting::find($check_program_slug->id);
            if($settingdata){
                $data['status'] = Helpers::STATUS_ENABLE;
                $update = Program_Setting::update($settingdata->id, $data);
                if($update){
                     //Log recent activity
                     Helpers::addToLog('Enable program setting', $program_slug, $data, $settingdata);
                    return response()->json(['message'=>'Program Enabled Successfully', 'status'=> 1, 'status_code'=> Helpers::STATUSCODE_SUCCESS]);
                }else{
                    return response()->json(['message'=> 'Error', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_FAILED]);
                }
            }elseif($settingdata->status == 1){
                return response()->json(['message'=> 'Program is currently active', 'status'=> 0, 'status_code'=>Helpers::STATUSCODE_FAILED]);
            }else{
                return response()->json(['message'=> 'Program setting not found', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_NOT_FOUND]);
            }
           
        }
    }
}