<?php

namespace App\Http\Controllers;

use App\Helpers;
use App\Models\Program;
use Illuminate\Http\Request;

class ProgramController extends Controller
{
    protected $perPage = 20;
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        //
    }

    public function all(Request $request, $status = 1)
    {
        if(isset($request['limit'])){
            $limit = $request['limit'];
        }else{
            $limit = $this->perPage;
        }
        $programs = Program::all($status, $limit);
        return response()->json($programs);
    }

    public function store(Request $request)
    {
        $this->validate($request, [
            'program_name'=> 'required|string|max:255',
            'program_slug' => 'required|alpha_num|max:255',
            'token'=> 'required',
        ]);
        
        $programslug = Helpers::createSlug($request['program_slug'], 'programs');
        if($programslug == false){
            return response()->json(['message'=> 'Program identifier already exists', 'status'=> 0, 'status_code'=> 400], 400);
         }
        $data = [
            'program_name' => $request['program_name'],
            'slug' => $programslug,
            'token'=> $request['token'],
        ];

        $store = Program::create($data);
        if($store){
            Helpers::addToLog('new Loyalty program added', $programslug, $data);
            $newprogram = Helpers::lastinserted('programs');
            return response()->json(['message'=> 'Program created successfully', 'status'=> 1, 'status_code'=> Helpers::STATUSCODE_SUCCESS, 'data'=> $newprogram]);
        }else{
            return response()->json(['message'=> 'Invaild request', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_FAILED]);
        }
    }

    public function show($slug)
    {
        $check_slug = Helpers::get_slug_id('programs', $slug);
        if(is_null($check_slug)){
            return response()->json(['message' => 'Program not found', 'status'=> 0, 'status'=>0, 'status_code'=> Helpers::STATUSCODE_NOT_FOUND]);
        }else{
            $program = Program::find($check_slug->id);
            return response()->json($program);
        }
    }

    public function update(Request $request, $slug)
    {
        $check_slug = Helpers::get_slug_id('programs', $slug);
        if(is_null($check_slug)){
            return response()->json(['message' => 'Program not found', 'status'=> 0,  'status_code'=> Helpers::STATUSCODE_NOT_FOUND]);
        }elseif($check_slug->status == 2){
            return response()->json(['message' => 'Program disabled', 'status'=>0, 'status_code'=> Helpers::STATUSCODE_FAILED ]);
        }else{
           $update_program = Program::update($check_slug->id, $request->only(['program_name', 'token']));
           if($update_program){
               $program = Program::find($check_slug->id);
               Helpers::addToLog('Update Loyalty program', $slug, $request->only(['program_name','token']), $check_slug);
               if($program){
                return response()->json(['message'=> 'Program updated successfully', 'status'=> 1, 'status_code'=> Helpers::STATUSCODE_SUCCESS, 'data'=> $program]);
               }else{
                return response()->json(['message'=> 'Invalid request', 'status'=> 0, 'status_code'=> Helpers::STATUSCODE_FAILED]);
               }
              
           }
        }
    }

    public function enable($slug)
    {
        $check_slug = Helpers::get_slug_id('programs', $slug);
        if(is_null($check_slug)){
            return response()->json(['message' => 'Program not found', 'status'=> 0,  'status_code'=> Helpers::STATUSCODE_NOT_FOUND]);
        }elseif($check_slug->status == 1){
            return response()->json(['message'=> 'Program currently active', 'status'=> 1, 'status_code'=> Helpers::STATUSCODE_SUCCESS]);
        }else{
            $data['status'] = Helpers::STATUS_ENABLE;
            $enable = Program::update($check_slug->id, $data);
            if($enable){
                //$program = Program::find($check_slug->id);
                Helpers::addToLog('Enable a Loyalty program',$slug,$data, $check_slug);
                return response()->json(['message'=> 'Program Enabled', 'status'=> 1, 'status_code'=> Helpers::STATUSCODE_SUCCESS]);
            }
        }   
    }

    public function disable($slug)
    {
        $check_slug = Helpers::get_slug_id('programs', $slug);
        if(is_null($check_slug)){
            return response()->json(['message' => 'Program not found', 'status'=> 0,  'status_code'=> Helpers::STATUSCODE_NOT_FOUND]);
        }elseif($check_slug->status == 2){
            return response()->json(['message'=> 'Program currently not active', 'status'=> 1, 'status_code'=> Helpers::STATUSCODE_SUCCESS]);
        }else{
            $data['status'] = Helpers::STATUS_DISABLE;
            $enable = Program::update($check_slug->id, $data);
            if($enable){
                //$program = Program::find($check_slug->id);
                Helpers::addToLog('Disable a Loyalty program',$slug,$data, $check_slug);
                return response()->json(['message'=> 'Program Disabled', 'status'=> 1, 'status_code'=> Helpers::STATUSCODE_SUCCESS]);
            }
        }   
    }
}
