<?php

namespace App\Jobs;

use App\Helpers;
use App\Models\Location;
use Carbon\Carbon;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Request;

class SearchLocation extends Job implements ShouldQueue
{

    protected $lat, $long, $token;
    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct()
    {
        //$this->token = $token;
        // $this->lat = $lat;
        // $this->long = $long;
       
    }


    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {
           $request = Helpers::lastinserted('users_request');
             $program_locations = DB::table('locations')
                                        ->where('program_id', $request->program_id)
                                        ->where('status',1)
                                        ->get();
       
       $location_results = $this->RadiusBaseSearch($program_locations, $request->latitude, $request->longitude);
       $location_message = $this->get_location_message($location_results);
       //dd($location_message);
    //    $message_priority = array_column($location_message, 'priority');
    //    dd($message_priority);
    //    $max_priority = max($message_priority);
    //    dd($max_priority);

       if(empty($location_results)){
            echo "No location found";
       }else{
            //check if a message has been sent to the membership_number
            $check_user = $this->check_user($request->membership_number);
            if($check_user){
                //check if a medium was used
                $lastmsg_details = $this->get_medium($request->membership_number);
                $priority_check = $this->next_priority($lastmsg_details->location_id, $lastmsg_details->priority);
                dd(is_null($priority_check));
                $check_interval = $this->check_interval($request->membership_number, $lastmsg_details->location_id);
                dd($check_interval);
                
            }else{
               
            }
            //get messages associated the location
            $msg = $this->get_location_message($location_results); 

            $check_user = $this->check_user($request->membership_number, $msg);
            
           
       }

        
    }

    /**
     * Search for "program" locations 
     * @param locations program locations in the database
     * @param lat user latitude
     * @param long user longitude
     * @param radius user radius
     * 
     * 
     * @return float in KM
     */
    private function RadiusBaseSearch($locations, $lat, $long)
    {

        $results = [];
        foreach ($locations as $loc) {
            $R = 6371;
            $dLat = deg2rad($lat - $loc->latitude);
            $dLon = deg2rad($long - $loc->longitude);
            $angle = sin($dLat/2) * sin($dLat/2) + cos(deg2rad($lat)) * cos(deg2rad($loc->latitude)) * sin($dLon/2) * sin($dLon/2);
            $c = 2 * atan2(sqrt($angle), sqrt(1-$angle));
            $d = $R * $c; //Distance in KM
             if($d <= $loc->radius){
                array_push($results, $loc);
            }
           
        }
        return $results;
    }

    public function get_location_message($locations)
    {
        $messages = [];
        foreach ($locations as $loc) {
            $message = DB::table('messages')->where('location_id', $loc->id)->get();
            array_push($messages, $message);
        }

        return $messages;
    }

    //checks if a message has been sent to the membership_number
    private function check_user($membership_number)
    {
        return DB::table('sent_messages')->where('member_number', $membership_number)->exists();
    }

    /**
     * 
     * @return message_medium
     * @return message_priority
     * of the last message sent to the membership_number 
     */
    private function get_medium($membership_number)
    {
       $get_medium = DB::table('messages')
                            ->join('sent_messages', function($join) use ($membership_number) {
                                $join->on('messages.id', '=', 'sent_messages.message_id')
                                        ->where('sent_messages.member_number', $membership_number);
                            })
                            ->select('messages.message_medium', 'messages.priority', 'messages.location_id')
                            ->first();
      return $get_medium;
    }

    private function check_interval($membership_number, $location_id)
    {
        $message_id = $this->last_sentMessage($membership_number);
        $details = DB::table('sent_messages')->where('id', $message_id)->first();//previously sent message
        $set_interval = DB::table('locations')->where('id', $location_id)->first();
        $dt = Carbon::parse($details->created_at);
        $time_interval = $dt->addHours($set_interval->message_interval);
        if($details->created_at < $time_interval){
            return false;
        }else{
            return true;
        }
    }

    private function last_sentMessage($membership_number){
        return DB::table('sent_messages')->where('member_number', $membership_number)->max('id');
    }

    //checks if a location with the next priority exists
    private function next_priority($location_id, $last_priority)
    {
        $next_priority = intval($last_priority + 1);
        return DB::table('messages')->where('location_id', $location_id)
                                        ->where('priority', $next_priority)
                                        ->first();
    }

    private function sent_message($message)
    {
       
    }



    
}
